from datetime import datetime

import aiohttp


async def my_func(name) -> str:
    """Get message from remote server."""
    async with aiohttp.ClientSession() as session:
        async with session.get("https://example.com/api.json", params={"name": name}) as response:
            data = await response.json()
            return data["message"]


def now() -> datetime:
    """Ready to mock method for date extraction."""
    return datetime.now()


def get_time() -> str:
    """Return today's date."""
    date = now()
    return f"Today is {date.strftime('%A, %B %d, %Y')}"



import argparse
import asyncio
import sys

from pydantic import ValidationError

from . import __version__
from .settings import Settings


async def do_sample_command_1():
    """Run used defined command 1."""


async def do_sample_command_2():
    """Run user defined command 2."""


def main():
    parser = argparse.ArgumentParser(
        description="cat for jupyter notebooks",
        argument_default=argparse.SUPPRESS,
    )
    parser.add_argument(
        "--list_field", help="Example of multi value list separated by comma.", type=str
    )
    parser.add_argument(
        "--version",
        help="Print version information and quite",
        action="version",
        version=__version__,
    )

    # Commands
    commands = parser.add_subparsers(title="Commands", dest="command")

    # Sample command 1
    sample_command_1 = commands.add_parser(
        "sample_command_1",
        help="Sample command description.",
        argument_default=argparse.SUPPRESS,
    )

    sample_command_1.set_defaults(func=do_sample_command_1)

    # Sample command 2
    sample_command_2 = commands.add_parser(
        "sample_command_2",
        help="Sample command description.",
        argument_default=argparse.SUPPRESS,
    )

    sample_command_2.set_defaults(func=do_sample_command_2)

    try:
        args = parser.parse_args()
        settings = Settings(**vars(args))
    except ValidationError as e:
        error = e.errors(include_url=False, include_context=False)[0]
        sys.exit(
            "Wrong argument value passed ({}): {}".format(
                error.get("loc", ("system",))[0], error.get("msg")
            )
        )

    if args.command:
        asyncio.run(args.func(settings))
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



# Sample test suite to show different scenarios of testing async methods
from datetime import datetime

import pytest
from aioresponses import aioresponses

from nbcat.nbcat import get_time, my_func


@pytest.mark.asyncio()
@pytest.mark.parametrize(
    "name,expected",
    [
        ("Alice", "Hello, Alice!"),
        ("Bob", "Hello, Bob!"),
        ("Charlie", "Hello, Charlie!"),
    ],
)
async def test_remote_api_mock(name, expected):
    with aioresponses() as mock:
        mock.get(
            f"https://example.com/api.json?name={name}",
            payload={"message": f"Hello, {name}!"},
            status=200,
        )
        resp = await my_func(name)
        assert resp == expected


def test_method_mock(mocker):
    fixed_date = datetime(1991, 2, 20)

    # It is always python birthday
    mocker.patch("nbcat.nbcat.now", return_value=fixed_date)

    assert get_time() == "Today is Wednesday, February 20, 1991"



from pydantic import BaseModel, model_validator


class Settings(BaseModel):
    """Validates cli arguments."""

    field: str | None = None
    list_field: list[str] = []
    bool_field: bool = False

    @model_validator(mode="before")
    def parse_list_field(values: dict):
        """You can pass multiple values as a comma separated string."""
        if isinstance(values.get("list_field"), str):
            values["list_field"] = values.get("list_field").split(",")
        return values
